@@ -61,7 +61,7 @@ void (*cbfatalfunc)(const char *message) = NULL;
 /* Allocate a region on memory. */
 void *cbmalloc(size_t size){
   char *p;
-  assert(size > 0);
+  assert(size > 0 && size < INT_MAX);
   if(!(p = malloc(size))){
     if(cbfatalfunc){
       cbfatalfunc("out of memory");
@@ -236,11 +236,11 @@ int cbstricmp(const char *astr, const char *bstr){
 
 /* Check whether a string begins with a key. */
 int cbstrfwmatch(const char *str, const char *key){
-  int len, i;
   assert(str && key);
-  len = strlen(key);
-  for(i = 0; i < len; i++){
-    if(str[i] != key[i] || str[i] == '\0') return FALSE;
+  while(*key != '\0'){
+    if(*str != *key || *str == '\0') return FALSE;
+    key++;
+    str++;
   }
   return TRUE;
 }
@@ -248,15 +248,17 @@ int cbstrfwmatch(const char *str, const char *key){
 
 /* Check whether a string begins with a key, with case insensitive evaluation. */
 int cbstrfwimatch(const char *str, const char *key){
-  int len, i, sc, kc;
+  int sc, kc;
   assert(str && key);
-  len = strlen(key);
-  for(i = 0; i < len; i++){
-    sc = str[i];
+  while(*key != '\0'){
+    if(*str == '\0') return FALSE;
+    sc = *str;
     if(sc >= 'A' && sc <= 'Z') sc += 'a' - 'A';
-    kc = key[i];
+    kc = *key;
     if(kc >= 'A' && kc <= 'Z') kc += 'a' - 'A';
-    if(sc != kc || sc == '\0') return FALSE;
+    if(sc != kc) return FALSE;
+    key++;
+    str++;
   }
   return TRUE;
 }
@@ -314,7 +316,7 @@ char *cbstrtolower(char *str){
 }
 
 
-/* Cut the space characters at head or tail of a string. */
+/* Cut space characters at head or tail of a string. */
 char *cbstrtrim(char *str){
   char *wp;
   int i, head;
@@ -337,6 +339,34 @@ char *cbstrtrim(char *str){
 }
 
 
+/* Squeeze space characters in a string and trim it. */
+char *cbstrsqzspc(char *str){
+  char *wp;
+  int i, spc;
+  assert(str);
+  wp = str;
+  spc = TRUE;
+  for(i = 0; str[i] != '\0'; i++){
+    if(str[i] > 0 && str[i] <= ' '){
+      if(!spc) *(wp++) = str[i];
+      spc = TRUE;
+    } else {
+      *(wp++) = str[i];
+      spc = FALSE;
+    }
+  }
+  *wp = '\0';
+  for(wp--; wp >= str; wp--){
+    if(*wp > 0 && *wp <= ' '){
+      *wp = '\0';
+    } else {
+      break;
+    }
+  }
+  return str;
+}
+
+
 /* Get a datum handle. */
 CBDATUM *cbdatumopen(const char *ptr, int size){
   CBDATUM *datum;
@@ -1266,8 +1296,11 @@ CBLIST *cbsplit(const char *ptr, int size, const char *delim){
 char *cbreadfile(const char *name, int *sp){
   int fd, size, rv;
   char iobuf[CB_IOBUFSIZ], *buf;
-  assert(name);
-  if((fd = open(name, O_RDONLY, 0)) == -1) return NULL;
+  if(name){
+    if((fd = open(name, O_RDONLY, 0)) == -1) return NULL;
+  } else {
+    fd = 0;
+  }
   buf = cbmalloc(1);
   size = 0;
   while((rv = read(fd, iobuf, CB_IOBUFSIZ)) > 0){
@@ -1288,9 +1321,13 @@ char *cbreadfile(const char *name, int *sp){
 /* Write data of a region into a file. */
 int cbwritefile(const char *name, const char *ptr, int size){
   int fd, err, wb;
-  assert(name && ptr);
+  assert(ptr);
   if(size < 0) size = strlen(ptr);
-  if((fd = open(name, O_WRONLY | O_CREAT | O_TRUNC, CB_FILEMODE)) == -1) return FALSE;
+  if(name){
+    if((fd = open(name, O_WRONLY | O_CREAT | O_TRUNC, CB_FILEMODE)) == -1) return FALSE;
+  } else {
+    fd = 1;
+  }
   err = FALSE;
   wb = 0;
   do {
@@ -1315,7 +1352,6 @@ CBLIST *cbreadlines(const char *name){
   int vsiz;
   CBMAP *pairs;
   CBLIST *list;
-  assert(name);
   if(!(buf = cbreadfile(name, NULL))) return NULL;
   pairs = cbmapopenex(3);
   cbmapput(pairs, "\r\n", 2, "\n", 1, TRUE);
@@ -2016,39 +2052,41 @@ CBLIST *cbxmlbreak(const char *str, int cr){
     if(str[i] == '\0'){
       if(i > pv) cblistpush(list, str + pv, i - pv);
       break;
-    } else if(cbstrfwimatch(str + i, "<!--")){
-      if(i > pv) cblistpush(list, str + pv, i - pv);
-      if((ep = strstr(str + i, "-->")) != NULL){
-        if(!cr) cblistpush(list, str + i, ep - str - i + 3);
-        i = ep - str + 2;
-        pv = i + 1;
-      }
-    } else if(cbstrfwimatch(str + i, "<![CDATA[")){
-      if(i > pv) cblistpush(list, str + pv, i - pv);
-      if((ep = strstr(str + i, "]]>")) != NULL){
-        i += 9;
-        datum = cbdatumopen(NULL, 0);
-        while(str + i < ep){
-          if(str[i] == '&'){
-            cbdatumcat(datum, "&amp;", 5);
-          } else if(str[i] == '<'){
-            cbdatumcat(datum, "&lt;", 4);
-          } else if(str[i] == '>'){
-            cbdatumcat(datum, "&gt;", 4);
-          } else {
-            cbdatumcat(datum, str + i, 1);
-          }
-          i++;
-        }
-        if(cbdatumsize(datum) > 0) cblistpush(list, cbdatumptr(datum), cbdatumsize(datum));
-        cbdatumclose(datum);
-        i = ep - str + 2;
-        pv = i + 1;
-      }
     } else if(!tag && str[i] == '<'){
-      if(i > pv) cblistpush(list, str + pv, i - pv);
-      tag = TRUE;
-      pv = i;
+      if(str[i+1] == '!' && str[i+2] == '-' && str[i+3] == '-'){
+        if(i > pv) cblistpush(list, str + pv, i - pv);
+        if((ep = strstr(str + i, "-->")) != NULL){
+          if(!cr) cblistpush(list, str + i, ep - str - i + 3);
+          i = ep - str + 2;
+          pv = i + 1;
+        }
+      } else if(str[i+1] == '!' && str[i+2] == '[' && cbstrfwimatch(str + i, "<![CDATA[")){
+        if(i > pv) cblistpush(list, str + pv, i - pv);
+        if((ep = strstr(str + i, "]]>")) != NULL){
+          i += 9;
+          datum = cbdatumopen(NULL, 0);
+          while(str + i < ep){
+            if(str[i] == '&'){
+              cbdatumcat(datum, "&amp;", 5);
+            } else if(str[i] == '<'){
+              cbdatumcat(datum, "&lt;", 4);
+            } else if(str[i] == '>'){
+              cbdatumcat(datum, "&gt;", 4);
+            } else {
+              cbdatumcat(datum, str + i, 1);
+            }
+            i++;
+          }
+          if(cbdatumsize(datum) > 0) cblistpush(list, cbdatumptr(datum), cbdatumsize(datum));
+          cbdatumclose(datum);
+          i = ep - str + 2;
+          pv = i + 1;
+        }
+      } else {
+        if(i > pv) cblistpush(list, str + pv, i - pv);
+        tag = TRUE;
+        pv = i;
+      }
     } else if(tag && str[i] == '>'){
       if(i > pv) cblistpush(list, str + pv, i - pv + 1);
       tag = FALSE;
@@ -2334,11 +2372,13 @@ char *cbdatestrhttp(time_t t, int jl){
 
 /* Get the time value of a date string in decimal, W3CDTF, or RFC 1123. */
 time_t cbstrmktime(const char *str){
+  const char *crp;
   char *pv, *rp;
-  int len;
+  int len, clen;
   time_t t;
   struct tm ts;
   assert(str);
+  if(cbstrfwimatch(str, "0x")) return (time_t)strtol(str + 2, NULL, 16);
   memset(&ts, 0, sizeof(struct tm));
   ts.tm_year = 70;
   ts.tm_mon = 0;
@@ -2349,8 +2389,8 @@ time_t cbstrmktime(const char *str){
   ts.tm_isdst = 0;
   len = strlen(str);
   t = (time_t)strtol(str, &pv, 10);
-  if(*pv == '\0' && (t < 1900 || t > 2039)) return t;
-  if(len == 4 || (len > 4 && str[4] == '-')){
+  if(*pv == '\0') return t;
+  if(len > 4 && str[4] == '-'){
     ts.tm_year = atoi(str) - 1900;
     if((pv = strchr(str, '-')) != NULL && pv - str == 4){
       rp = pv + 1;
@@ -2379,68 +2419,120 @@ time_t cbstrmktime(const char *str){
     ts.tm_sec += cbjetlag();
     return mktime(&ts);
   }
-  if(len >= 25 && str[3] == ',' && str[4] == ' ' && str[7] == ' ' && str[11] == ' ' &&
-     str[16] == ' ' &&  str[19] == ':' && str[22] == ':'){
-    ts.tm_mday = atoi(str + 5);
-    if(cbstrfwmatch(str + 8, "Jan")){
-      ts.tm_mon = 0;
-    } else if(cbstrfwmatch(str + 8, "Feb")){
-      ts.tm_mon = 1;
-    } else if(cbstrfwmatch(str + 8, "Mar")){
-      ts.tm_mon = 2;
-    } else if(cbstrfwmatch(str + 8, "Apr")){
-      ts.tm_mon = 3;
-    } else if(cbstrfwmatch(str + 8, "May")){
-      ts.tm_mon = 4;
-    } else if(cbstrfwmatch(str + 8, "Jun")){
-      ts.tm_mon = 5;
-    } else if(cbstrfwmatch(str + 8, "Jul")){
-      ts.tm_mon = 6;
-    } else if(cbstrfwmatch(str + 8, "Aug")){
-      ts.tm_mon = 7;
-    } else if(cbstrfwmatch(str + 8, "Sep")){
-      ts.tm_mon = 8;
-    } else if(cbstrfwmatch(str + 8, "Oct")){
-      ts.tm_mon = 9;
-    } else if(cbstrfwmatch(str + 8, "Nov")){
-      ts.tm_mon = 10;
-    } else if(cbstrfwmatch(str + 8, "Dec")){
-      ts.tm_mon = 11;
+  if(len > 4 && str[4] == '/'){
+    ts.tm_year = atoi(str) - 1900;
+    if((pv = strchr(str, '/')) != NULL && pv - str == 4){
+      rp = pv + 1;
+      ts.tm_mon = atoi(rp) - 1;
+      if((pv = strchr(rp, '/')) != NULL && pv - str == 7){
+        rp = pv + 1;
+        ts.tm_mday = atoi(rp);
+        if((pv = strchr(rp, ' ')) != NULL && pv - str == 10){
+          rp = pv + 1;
+          ts.tm_hour = atoi(rp);
+          if((pv = strchr(rp, ':')) != NULL && pv - str == 13){
+            rp = pv + 1;
+            ts.tm_min = atoi(rp);
+          }
+          if((pv = strchr(rp, ':')) != NULL && pv - str == 16){
+            rp = pv + 1;
+            ts.tm_sec = atoi(rp);
+          }
+          if((pv = strchr(rp, '.')) != NULL && pv - str == 19) rp = pv + 1;
+          strtol(rp, &pv, 10);
+          if((*pv == '+' || *pv == '-') && strlen(pv) == 6 && pv[3] == ':')
+            ts.tm_sec -= (atoi(pv + 1) * 3600 + atoi(pv + 4) * 60) * (pv[0] == '+' ? 1 : -1);
+        }
+      }
     }
-    ts.tm_year = atoi(str + 12) - 1900;
-    ts.tm_hour = atoi(str + 17);
-    ts.tm_min = atoi(str + 20);
-    ts.tm_sec = atoi(str + 23);
-    if(len == 31 && (str[26] == '+' || str[26] == '-')){
-      ts.tm_sec -= ((str[27] - '0') * 36000 + (str[28] - '0') * 3600 +
-                    (str[29] - '0') * 600 + (str[30] - '0') * 60) * (str[26] == '+' ? 1 : -1);
-    } else if(len > 26){
-      if(!strcmp(str + 26, "JST")){
-        ts.tm_sec -= 9 * 3600;
-      } else if(!strcmp(str + 26, "CCT")){
-        ts.tm_sec -= 8 * 3600;
-      } else if(!strcmp(str + 26, "KST")){
-        ts.tm_sec -= 9 * 3600;
-      } else if(!strcmp(str + 26, "EDT")){
-        ts.tm_sec -= -4 * 3600;
-      } else if(!strcmp(str + 26, "EST")){
-        ts.tm_sec -= -5 * 3600;
-      } else if(!strcmp(str + 26, "CDT")){
-        ts.tm_sec -= -5 * 3600;
-      } else if(!strcmp(str + 26, "CST")){
-        ts.tm_sec -= -6 * 3600;
-      } else if(!strcmp(str + 26, "MDT")){
-        ts.tm_sec -= -6 * 3600;
-      } else if(!strcmp(str + 26, "MST")){
-        ts.tm_sec -= -7 * 3600;
-      } else if(!strcmp(str + 26, "PDT")){
-        ts.tm_sec -= -7 * 3600;
-      } else if(!strcmp(str + 26, "PST")){
-        ts.tm_sec -= -8 * 3600;
-      } else if(!strcmp(str + 26, "HDT")){
-        ts.tm_sec -= -9 * 3600;
-      } else if(!strcmp(str + 26, "HST")){
-        ts.tm_sec -= -10 * 3600;
+    ts.tm_sec += cbjetlag();
+    return mktime(&ts);
+  }
+  crp = str;
+  if(len >= 4 && str[3] == ',') crp = str + 4;
+  while(*crp == ' '){
+    crp++;
+  }
+  ts.tm_mday = atoi(crp);
+  while((*crp >= '0' && *crp <= '9') || *crp == ' '){
+    crp++;
+  }
+  if(cbstrfwimatch(crp, "Jan")){
+    ts.tm_mon = 0;
+  } else if(cbstrfwimatch(crp, "Feb")){
+    ts.tm_mon = 1;
+  } else if(cbstrfwimatch(crp, "Mar")){
+    ts.tm_mon = 2;
+  } else if(cbstrfwimatch(crp, "Apr")){
+    ts.tm_mon = 3;
+  } else if(cbstrfwimatch(crp, "May")){
+    ts.tm_mon = 4;
+  } else if(cbstrfwimatch(crp, "Jun")){
+    ts.tm_mon = 5;
+  } else if(cbstrfwimatch(crp, "Jul")){
+    ts.tm_mon = 6;
+  } else if(cbstrfwimatch(crp, "Aug")){
+    ts.tm_mon = 7;
+  } else if(cbstrfwimatch(crp, "Sep")){
+    ts.tm_mon = 8;
+  } else if(cbstrfwimatch(crp, "Oct")){
+    ts.tm_mon = 9;
+  } else if(cbstrfwimatch(crp, "Nov")){
+    ts.tm_mon = 10;
+  } else if(cbstrfwimatch(crp, "Dec")){
+    ts.tm_mon = 11;
+  } else {
+    ts.tm_mon = -1;
+  }
+  if(ts.tm_mon >= 0) crp += 3;
+  while(*crp == ' '){
+    crp++;
+  }
+  ts.tm_year = atoi(crp);
+  if(ts.tm_year >= 1969) ts.tm_year -= 1900;
+  while(*crp >= '0' && *crp <= '9'){
+    crp++;
+  }
+  while(*crp == ' '){
+    crp++;
+  }
+  if(ts.tm_mday > 0 && ts.tm_mon >= 0 && ts.tm_year >= 0){
+    clen = strlen(crp);
+    if(clen >= 8 && crp[2] == ':' && crp[5] == ':'){
+      ts.tm_hour = atoi(crp + 0);
+      ts.tm_min = atoi(crp + 3);
+      ts.tm_sec = atoi(crp + 6);
+      if(clen == 14 && crp[8] == ' ' && (crp[9] == '+' || crp[9] == '-')){
+        ts.tm_sec -= ((crp[10] - '0') * 36000 + (crp[11] - '0') * 3600 +
+                      (crp[12] - '0') * 600 + (crp[13] - '0') * 60) * (crp[9] == '+' ? 1 : -1);
+      } else if(clen > 9){
+        if(!strcmp(crp + 9, "JST")){
+          ts.tm_sec -= 9 * 3600;
+        } else if(!strcmp(crp + 9, "CCT")){
+          ts.tm_sec -= 8 * 3600;
+        } else if(!strcmp(crp + 9, "KST")){
+          ts.tm_sec -= 9 * 3600;
+        } else if(!strcmp(crp + 9, "EDT")){
+          ts.tm_sec -= -4 * 3600;
+        } else if(!strcmp(crp + 9, "EST")){
+          ts.tm_sec -= -5 * 3600;
+        } else if(!strcmp(crp + 9, "CDT")){
+          ts.tm_sec -= -5 * 3600;
+        } else if(!strcmp(crp + 9, "CST")){
+          ts.tm_sec -= -6 * 3600;
+        } else if(!strcmp(crp + 9, "MDT")){
+          ts.tm_sec -= -6 * 3600;
+        } else if(!strcmp(crp + 9, "MST")){
+          ts.tm_sec -= -7 * 3600;
+        } else if(!strcmp(crp + 9, "PDT")){
+          ts.tm_sec -= -7 * 3600;
+        } else if(!strcmp(crp + 9, "PST")){
+          ts.tm_sec -= -8 * 3600;
+        } else if(!strcmp(crp + 9, "HDT")){
+          ts.tm_sec -= -9 * 3600;
+        } else if(!strcmp(crp + 9, "HST")){
+          ts.tm_sec -= -10 * 3600;
+        }
       }
     }
     ts.tm_sec += cbjetlag();
