@@ -219,12 +219,18 @@ char *cbstrtoupper(char *str);
 char *cbstrtolower(char *str);
 
 
-/* Cut the space characters at head or tail of a string.
+/* Cut space characters at head or tail of a string.
    `str' specifies the pointer of a string to convert.
    The return value is the pointer to the string. */
 char *cbstrtrim(char *str);
 
 
+/* Squeeze space characters in a string and trim it.
+   `str' specifies the pointer of a string to convert.
+   The return value is the pointer to the string. */
+char *cbstrsqzspc(char *str);
+
+
 /* Get a datum handle.
    `ptr' specifies the pointer to the region of the initial content.  If it is `NULL', an empty
    datum is created.
@@ -611,7 +617,7 @@ CBLIST *cbsplit(const char *ptr, int size, const char *delim);
 
 
 /* Read whole data of a file.
-   `name' specifies the name of a file.
+   `name' specifies the name of a file.  If it is `NULL', the standard input is specified.
    `sp' specifies the pointer to a variable to which the size of the region of the return
    value is assigned.  If it is `NULL', it is not used.
    The return value is the pointer to the allocated region of the read data.  Because an
@@ -623,7 +629,7 @@ char *cbreadfile(const char *name, int *sp);
 
 
 /* Write a serial datum into a file.
-   `name specifies the name of a file.
+   `name specifies the name of a file.  If it is `NULL', the standard output is specified.
    `ptr' specifies the pointer to the region of the source content.
    `size' specifies the size of the region.  If it is negative, the size is assigned with
    `strlen(ptr)'.
@@ -633,15 +639,18 @@ int cbwritefile(const char *name, const char *ptr, int size);
 
 
 /* Read every line of a file.
-   `name' specifies the name of a file.
-   The return value is a list handle of the lines if successful, else it is NULL.
-   Line separators are cut out. */
+   `name' specifies the name of a file.  If it is `NULL', the standard input is specified.
+   The return value is a list handle of the lines if successful, else it is NULL.  Line
+   separators are cut out.  Because the handle of the return value is opened with the function
+   `cblistopen', it should be closed with the function `cblistclose' if it is no longer in use. */
 CBLIST *cbreadlines(const char *name);
 
 
 /* Read names of files in a directory.
    `name' specifies the name of a directory.
-   The return value is a list handle of names if successful, else it is NULL. */
+   The return value is a list handle of names if successful, else it is NULL.
+   Because the handle of the return value is opened with the function `cblistopen', it should
+   be closed with the function `cblistclose' if it is no longer in use. */
 CBLIST *cbdirlist(const char *name);
 
 
@@ -978,8 +987,8 @@ char *cbdatestrwww(time_t t, int jl);
 char *cbdatestrhttp(time_t t, int jl);
 
 
-/* Get the time value of a date string in decimal, W3CDTF, or RFC 1123.
-   `str' specifies a date string in decimal, W3CDTF, or RFC 1123.
+/* Get the time value of a date string in decimal, hexadecimal, W3CDTF, or RFC 822 (1123).
+   `str' specifies a date string in decimal, hexadecimal, W3CDTF, or RFC 822 (1123).
    The return value is the time value of the date or -1 if the format is invalid. */
 time_t cbstrmktime(const char *str);
 
@@ -1026,6 +1035,12 @@ CBMAP *cbmapopenex(int bnum);
 #define CB_LISTVAL(list,index,sp) ((const char *)(list->array[list->start+index].dptr))
 
 
+/* Alias of `cblistval'.
+   However, `sp' is needed. */
+#define CB_LISTVAL2(list,index,sp) (*sp = list->array[list->start+index].dsize,\
+  (const char *)(list->array[list->start+index].dptr))
+
+
 
 #if defined(__cplusplus)                 /* export for C++ */
 }
